# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

* [`zookeeperd`](#zookeeperd): Main class of puppet module configuring nodes of an zookeeper ensamble.
* [`zookeeperd::config`](#zookeeperdconfig): control the configuration of the node
* [`zookeeperd::install`](#zookeeperdinstall): control the configuration of the node
* [`zookeeperd::service`](#zookeeperdservice): control the configuration of the node

### Defined types

* [`zookeeperd::node`](#zookeeperdnode): A short summary of the purpose of this defined type.

## Classes

### <a name="zookeeperd"></a>`zookeeperd`

zookeeperd

Puppet module to configure the nodes of an zookeeper ensamble.

#### Examples

##### 

```puppet
include zookeeperd
```

##### 

```puppet
zookeeperd::ensamble: 'Awesamly important, dynamically growing zookeeper cluster'
zookeeperd::enable_autoconfig: true
```

#### Parameters

The following parameters are available in the `zookeeperd` class:

* [`ensure`](#ensure)
* [`manage_packages`](#manage_packages)
* [`package_names`](#package_names)
* [`data_dir`](#data_dir)
* [`user`](#user)
* [`group`](#group)
* [`cfg_path`](#cfg_path)
* [`config`](#config)
* [`maintenance_service`](#maintenance_service)
* [`maintenance_schedule`](#maintenance_schedule)
* [`maintenance_snapretention`](#maintenance_snapretention)
* [`manage_service`](#manage_service)
* [`service_name`](#service_name)
* [`service_enabled`](#service_enabled)
* [`service_running`](#service_running)
* [`ensamble`](#ensamble)
* [`nodes`](#nodes)
* [`myid`](#myid)
* [`nodename`](#nodename)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

add or remove this node from ensamble

Default value: `'present'`

##### <a name="manage_packages"></a>`manage_packages`

Data type: `Boolean`

enable puppet to manage package installation

Default value: ``true``

##### <a name="package_names"></a>`package_names`

Data type: `Array[String]`

list of packages to be installed

Default value: `[]`

##### <a name="data_dir"></a>`data_dir`

Data type: `Stdlib::Absolutepath`

path to the data directory of zookeeper

Default value: `''`

##### <a name="user"></a>`user`

Data type: `Variant[Integer,String]`

username of zookeeper service

Default value: `'zookeeper'`

##### <a name="group"></a>`group`

Data type: `Variant[Integer,String]`

groupname of zookeeper service

Default value: `'zookeeper'`

##### <a name="cfg_path"></a>`cfg_path`

Data type: `Stdlib::Absolutepath`

path to configuration directory

Default value: `''`

##### <a name="config"></a>`config`

Data type: `Hash[String, Scalar]`

parameter to generate zoo.cfg configration file

Default value: `{}`

##### <a name="maintenance_service"></a>`maintenance_service`

Data type: `Boolean`

establish the systemd units for maintenance cleanup service and its timer

Default value: ``true``

##### <a name="maintenance_schedule"></a>`maintenance_schedule`

Data type: `Array[String]`

array for timer wallclocks to schedule maintenance clienups, refer to
[systemd.time(7)#Parsing Timestamps](https://www.freedesktop.org/software/systemd/man/systemd.time.html#Parsing%20Timestamps) for details on the items of this array

Default value: `[]`

##### <a name="maintenance_snapretention"></a>`maintenance_snapretention`

Data type: `Integer`

configure the number of snapshots to be retained during cleanup

Default value: `3`

##### <a name="manage_service"></a>`manage_service`

Data type: `Boolean`

enable puppet to manage the zookeeper serivce

Default value: ``true``

##### <a name="service_name"></a>`service_name`

Data type: `String`

name of the service to control

Default value: `'zookeeper'`

##### <a name="service_enabled"></a>`service_enabled`

Data type: `Boolean`

enable service at boot time

Default value: ``true``

##### <a name="service_running"></a>`service_running`

Data type: `Boolean`

ensure running of the service

Default value: ``true``

##### <a name="ensamble"></a>`ensamble`

Data type: `Optional[String]`

name of the ensamble

Default value: ``undef``

##### <a name="nodes"></a>`nodes`

Data type: `Hash`

list of nodes, if autoconfiguration disable

Default value: `{}`

##### <a name="myid"></a>`myid`

Data type: `Integer`

node id of zookeeper instance

Default value: `$facts['zookeeperd']['myid']`

##### <a name="nodename"></a>`nodename`

Data type: `String`

fqdn of this node

Default value: `$facts['networking']['fqdn']`

### <a name="zookeeperdconfig"></a>`zookeeperd::config`

zookeeperd::config

Internal class do not use of its own.

### <a name="zookeeperdinstall"></a>`zookeeperd::install`

zookeeperd::install

Internal class do not use of its own.

### <a name="zookeeperdservice"></a>`zookeeperd::service`

zookeeperd::service

Internal class do not use of its own.

## Defined types

### <a name="zookeeperdnode"></a>`zookeeperd::node`

zookeeperd::node

A node definition to be added as server line in zoo.cfg file.
One may define the nodes of an zookeeper ensamble by instanciating this resource.
The module itself is implementing a factory to create nodes to all keys added to
zookeeperd::nodes hash.
This resource is also used with autoconfiguration, which exports this resource and
collect all exported nodes of the same ensamble.

#### Examples

##### 

```puppet
zookeeperd::node { 'first node of this cluster':
  nodeid   => 1,
  nodename => 'first.zk.example.org',
}
zookeeperd::node { 'second node of this cluster':
  nodeid   => 2,
  nodename => 'second.zk.example.org',
}
```

#### Parameters

The following parameters are available in the `zookeeperd::node` defined type:

* [`nodeid`](#nodeid)
* [`nodename`](#nodename)
* [`ensure`](#ensure)
* [`cfgtgt`](#cfgtgt)
* [`ensamble`](#ensamble)
* [`leaderport`](#leaderport)
* [`electionport`](#electionport)

##### <a name="nodeid"></a>`nodeid`

Data type: `Integer`

node id (myid) of the zookeeper node to be added to this configuration

##### <a name="nodename"></a>`nodename`

Data type: `String`

fqdn or IP address of the node to be added

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

ensure meta-parameter to add or remove this node

Default value: `'present'`

##### <a name="cfgtgt"></a>`cfgtgt`

Data type: `String`

path to the configuration file to which this server should be added

Default value: `$zookeeperd::cfg_path`

##### <a name="ensamble"></a>`ensamble`

Data type: `Optional[String]`

name of the ensamble this node belongs to (used with autoconfig)

Default value: ``undef``

##### <a name="leaderport"></a>`leaderport`

Data type: `Integer`

leader port parameter of the server

Default value: `2888`

##### <a name="electionport"></a>`electionport`

Data type: `Integer`

port used for leader election

Default value: `3888`

